//$fa = 1 //is the minimum angle for a fragment. The default value is 12 (i.e. 30 fragments for a full circle).
//$fs = 0.4; //is the minimum size of a fragment. The default value is 2 so very small circles have a smaller number of fragments than specified using $fa.
$fn=30; // default=0 When this variable has a value greater than zero, the other two variables are ignored, and a full circle is rendered using this number of fragments. Don't set it over 100.

//Create variables
height = 80;
height_wood = 30;
height_acrylic = height - height_wood;

radius = 250;

/////// Hexagon dimensions
//The radius of the hexagon
hex_radius = round(radius * 0.65);
//The slightly larger radius for the glass plate
offset_hex_radius = hex_radius + 10;
//Depth of the glass plate
depth_glass_bottom = 5;

/////// Spirograph dimensions
spiro_scale = 2;
//Spirograph etching depth
etch_depth = 3;
//The spirograph etching takes a few seconds, deactivate it by setting this to false
etch_spiro = true;

/////// Cubby hole dimensions 
//The radius of the outside arc of the cubby hole
radius_hole = round(radius * 0.95);
//The distance of the inside straight edge of the cubby hole
inside_distance = hex_radius * sqrt(3)/2 + round(hex_radius * 0.175);
//The angle of the cubby holes
angle_cubby = 60 - 10;
//Total depth of the cubby holes
cubby_depth = 50;


//A little padding to not have z-index artifacts in the preview mode
nothing = 0.01;

/////////// Create model
acrylic_box();
glass_plate();
wooden_box();
//puzzle pieces - no

/////////// Separate Objects ///////////

//Acrylic bottom
module acrylic_box() {
    cubby_depth_acrylic = cubby_depth - height_wood;
    
    color([0.3,0.3,0.3,1])
    difference() {
        cylinder($fa=1, height_acrylic, r = radius, center = false);
        
        //Bigger hexagon to hold glass plate
        translate([0,0,height_acrylic - depth_glass_bottom]) {
            hexagon(depth_glass_bottom + nothing, offset_hex_radius);
        }//translate
        
        //Inner hexagon all the way to the bottom
        translate([0,0,-nothing]) {
            hexagon(height_acrylic + nothing, hex_radius);
        }//translate
            
        //Cut out the cubby holes
        //Move the bottom part up till the bottom of the cubby hole
        translate([0,0,height_acrylic - cubby_depth_acrylic]) {
            for ( i = [0 : 3] ){
                rotate(-i * 60) {
                    cubby_hole(angle = angle_cubby, r = radius_hole, h = cubby_depth_acrylic + 2*nothing, d  = inside_distance);
                }//rotate
            }//for i
        }//translate
                
    }//difference
}//module acrylic_bottom


//Glass plate
module glass_plate() {
    color([1,1,1,0.6])
    translate([0,0,height_acrylic - depth_glass_bottom]) {
        difference() {
            hexagon(depth_glass_bottom + nothing, offset_hex_radius);
            
            if(etch_spiro == true) {
                //Cut out a little from thhe spirograph
                translate([0,0,depth_glass_bottom - etch_depth - nothing]) {
                    spirograph(etch_depth + 2*nothing);
                }//translate
            }//if
        }//difference
            
    }//translate
}//module glass_plate


//Wooden top
module wooden_box() {
    color("brown")
    translate([0,0,height_acrylic]) {
        difference() {
            cylinder($fa=1, height_wood, r = radius, center = false);
            
            //Cut out more than the actual height, due to artifacts in preview mode, thus translate it a little down
            translate([0,0,-nothing]) {
                //Central hexagon
                hexagon(height_wood + 2* nothing, hex_radius);
                
                //Cut out the cubby holes
                for ( i = [0 : 3] ){
                    rotate(-i * 60) {
                        cubby_hole(angle = angle_cubby, r = radius_hole, h = height_wood + 2*nothing, d  = inside_distance);
                    }//rotate
                }//for i
            }//translate
            
            //Cut out a little of the spirograph from the top
            if(etch_spiro == true) {
                translate([0,0,height_wood - etch_depth - nothing]) {
                    spirograph(etch_depth + 2*nothing);
                }//translate
            }//if
        }//difference
    }//translate
}//module wooden_box

/////////// Cubby Hole Functions ///////////

module cubby_hole(angle = 60, r, h, d) {
    difference() {
        //Create pie slice, centered along the postive x-axis
        rotate(-angle/2) pie_slice(r, h, angle);
        //Remove a cube from it, centered on the origin
        translate([0,0,-nothing]) {
            linear_extrude(height= h + 2*nothing) 
                square(2*d, center=true); 
        }//translate
    }//difference
}//module cubby_hole

//Adjusted to also have a height from: https://3dprinting.stackexchange.com/questions/10638
module pie_slice(r=3.0, h=3.0, a=30) {     
   intersection() {
       cylinder(h=h,r=r,center=false);
       linear_extrude(height=h) square(r);
       rotate(a-90) linear_extrude(height=h) square(r); 
   }//intersection
}//module pie_slice

/////////// Helper Functions ///////////

module hexagon(h, r) {
    rotate(30) {
        cylinder($fn = 6, h, r = r, center = false);
    }
}//module hexagon

//Get the spirograph from the SVG
module spirograph(height) {
    linear_extrude(height = height) {
        scale(spiro_scale)
        import("puzzle_spirograph_outlined.svg");
    }
}//module spirograph

